
#include "stm32f1xx_hal.h"
#include "font6x8.h"
#include "lcd8544.h"
#include "main.h"

uint8_t lcd8544_buff[504]; //  

void lcd8544_sendcmd( SPI_HandleTypeDef *hspi, uint8_t cmd )//
{
	LCD_DC0;
	HAL_SPI_Transmit( hspi, &cmd, 1, 1000 );
}

//  
void lcd8544_clr( uint8_t data ) {

	for ( int x=0;x < 504; x++ ) lcd8544_buff[x] = data;
}

//    
void lcd8544_update( SPI_HandleTypeDef *hspi ) {
	uint8_t adr[] = { 0x40, 0x80 };
	LCD_DC0;            //  

	HAL_SPI_Transmit_DMA( hspi, adr, 2 );
	LCD_DC1;            //  

	HAL_SPI_Transmit_DMA( hspi, lcd8544_buff, 504 );
}

// 
void lcd8544_init( SPI_HandleTypeDef *hspi ) {

/*
0x21 -    
0xC1

0x06 -  ,  4  7

0x13 - Bias 0b0001 0xxx -   

0x20 -    
12 - 0b1100 - normal mode
		0b1101 - invert mode
		0b1001 -   
		0b1000 -  
*/

	uint8_t setup[] = {0x21, 0xc1, 0x06, 0x13, 0x20, 0x0c };

//  
	  LCD_RST0;           // RST=0 -  
		HAL_Delay( 1 );
	  LCD_RST1;           // RST=1

	  //   
	  LCD_DC0;            //  
		HAL_SPI_Transmit_DMA( hspi, setup, 6 );

	  lcd8544_update( hspi );
}

//  
void lcd8544_putpix(uint8_t x, uint8_t y, uint8_t mode) {
	if ((x>84) || (y>47)) return;

	unsigned int adr=(y>>3)*84+x;
	uint8_t data=(1<<(y&0x07));

	if (mode) lcd8544_buff[adr]|=data;
   else lcd8544_buff[adr]&=~data;
}

//   
void lcd8544_line(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2, uint8_t mode) {
signed char   dx, dy, sx, sy;
uint8_t  x,  y, mdx, mdy, l;


  dx=x2-x1; dy=y2-y1;

  if (dx>=0) { mdx=dx; sx=1; } else { mdx=x1-x2; sx=-1; }
  if (dy>=0) { mdy=dy; sy=1; } else { mdy=y1-y2; sy=-1; }

  x=x1; y=y1;

  if (mdx>=mdy) {
     l=mdx;
     while (l>0) {
         if (dy>0) { y=y1+mdy*(x-x1)/mdx; }
            else { y=y1-mdy*(x-x1)/mdx; }
         lcd8544_putpix(x,y,mode);
         x=x+sx;
         l--;
     }
  } else {
     l=mdy;
     while (l>0) {
        if (dy>0) { x=x1+((mdx*(y-y1))/mdy); }
          else { x=x1+((mdx*(y1-y))/mdy); }
        lcd8544_putpix(x,y,mode);
        y=y+sy;
        l--;
     }
  }
  lcd8544_putpix(x2, y2, mode);

}



//   ( )
void lcd8544_rect(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2, uint8_t mode) {
	lcd8544_line(x1,y1, x2,y1, mode);
	lcd8544_line(x1,y2, x2,y2, mode);
	lcd8544_line(x1,y1, x1,y2, mode);
	lcd8544_line(x2,y1, x2,y2, mode);
}

void lcd8544_char( uint8_t x, uint8_t row, uint8_t ch, uint8_t font )
{
	uint8_t px, py, fx, fy, offset;
	uint16_t font_idx;
	const uint8_t *fontp;
	
	switch( font )
	{
		case 1:
			fontp = MediumNumbers;
			fx = *fontp; fy = *( fontp+1 ); offset = *( fontp+2 );
		break;
		case 2:
			fontp = BigNumbers;
			fx = *fontp; fy = *( fontp+1 ); offset = *( fontp+2 );
		break;
		default:
			fontp = FontLAT;
			fx = *fontp; fy = *( fontp+1 ); offset = *( fontp+2 );
	}
	
	ch=ch-offset;
	font_idx = ch*fx*fy+4;
	for( py = 0 ; py < fy ; py++ )
	{
		for( px = 0 ; px < fx ; px++ )
		{
			lcd8544_buff[ x+row*84+px+py*84 ] = *( fontp + font_idx+px+py*fx );
		}
	}
}
//      
void lcd8544_putchar(uint8_t px, uint8_t py, uint8_t ch, uint8_t mode) {
	const uint8_t *fontpointer;


	if (ch<127) {	//  
		fontpointer=FontLAT+4; ch=ch-32; }
		else	{	//   
		fontpointer=FontRUS+4; ch=ch-192;}

    uint8_t lcd_YP=7- (py & 0x07);    //     
    uint8_t lcd_YC=(py & 0xF8)>>3; 	//     
    uint8_t x;
	for (x=0; x<6; x++) {

		uint8_t temp=*(fontpointer+ch*6+x);

		if (mode!=0) {
			temp=~temp;
			if (py>0) lcd8544_putpix(px, py-1, 1);	//      -   
		}

		temp&=0x7F;

		lcd8544_buff[lcd_YC*84+px]=lcd8544_buff[lcd_YC*84+px] | (temp<<(7-lcd_YP)); 	//    

	    if (lcd_YP<7) lcd8544_buff[(lcd_YC+1)*84+px]=lcd8544_buff[(lcd_YC+1)*84+px] | (temp>>(lcd_YP+1)); 	//    

		px++;
		if (px>83) return;
	}
}


//   
void lcd8544_dec( uint8_t x, uint8_t row, uint32_t n, uint8_t count, uint8_t font ) {
	uint16_t divid=10000;
	uint8_t i, fx;
	const uint8_t *fontp;

	switch( font )
	{
		case 1:
			fontp = MediumNumbers;
			fx = *fontp;
		break;
		case 2:
			fontp = BigNumbers;
			fx = *fontp;
		break;
		default:
			fontp = FontLAT;
			fx = *fontp;
	}

	for (i=5; i!=0; i--) {

		uint8_t res=n/divid;

		if (i<=count) {
			lcd8544_char(x, row, res+'0', font);
			x=x+fx;
		}

		n%=divid;
		divid/=10;
	}
}

//  
void lcd8544_putstr( uint8_t x, uint8_t y, uint8_t *str, uint8_t mode ) {
	if (mode) lcd8544_leftchline(x, y);
	while (*str!=0) {
		lcd8544_putchar(x, y, *str, mode);
		x=x+6;
		str++;
	}
}

//       
void lcd8544_leftchline( uint8_t x, uint8_t y) {
	if (x>0) lcd8544_line(x-1, y-1, x-1, y+6, 1);
}
